/***************************************************************************
 *            database-fun.c
 *
 *  Thu Oct 27 15:52:11 2005
 *  Copyright  2005  Tomek Jankowski
 *  Email	tomcio-j@o2.pl
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <glade/glade.h>
#include <gtk/gtk.h>
#include <glib.h>
#include <glib/gprintf.h>
#include <glib/gi18n-lib.h>
#include <sqlite3.h>

#include "database-fun.h"
#include "interface-fun.h"

#define PATH 1024
#define	DEFAULT_LENGHT	1024

/*
 *	Funkcja jest wykonywana w przypadku, gdy plik z baza danych nie zostanie 
 *	odnaleziony. Funkcja zajmuje sie TYLKO tworzeniem nowej bazy danych,
 *	konwersja pomiedzy roznymi wersjami baz danych jest wykonywana przez program
 *	'Gbiblioteka Converter'
 */

void		create_db			(void)	{
	sqlite3_stmt	*db_row;
	gint			status;
	gchar			command	[PATH];
	gchar 			db_path	[DEFAULT_LENGHT];
	
	g_printf ("This is first run of GLibrary, program must create base files, please wait...\n");
	set_statusbar_text (_("First run of GLibrary, creating base files..."));
	
	/*	Tworzymy katalog w ktorym zapiszemy baze daych, domysla sciezka to:
	 *	~/.config/gbiblioteka
	 */
	g_sprintf (command, "%s/.config", g_get_home_dir ());
	status =  g_mkdir (command, 0711);
	
	g_sprintf (command, "%s/.config/glibrary", g_get_home_dir ());
	status =  g_mkdir (command, 0711);
	
	g_sprintf (db_path, "%s/.config/glibrary/database-1rc", g_get_home_dir ());
	status = sqlite3_open (db_path, &database);
	g_printf ("<<< Opening data base, SQLite result: %d\n", status);

	status = sqlite3_prepare(database, "CREATE TABLE single_volume_books "
										 "(id INTEGER PRIMARY KEY AUTOINCREMENT, "
										 
										 "orginal_title TEXT, "
									 	 "title TEXT, "
										 
										 "first_author INTEGER, "
										 "second_author INTEGER, "
										 "third_author INTEGER, "
										 
										 "genere INTEGER, "
										 "publish_house INTEGER, "
										 
										 "first_translator INTEGER, "
										 "second_translator INTEGER, "
										 "third_translator INTEGER, "
										 
										 "language INTEGER, "
										 "orginal_language INTEGER, "
										 
										 "isbn INTEGER, "
										 "publish_year INTEGER, "
										 "pages INTEGER, "
										 
										 "description TEXT);",
										 -1, &db_row, 0);
	status = sqlite3_step (db_row);
	g_printf ("<<< Creating table \'books\', SQLite result: %d\n", status);
	sqlite3_finalize (db_row);
	
	status = sqlite3_prepare(database, "CREATE TABLE multi_volume_books_shared_data "
										 "(id INTEGER PRIMARY KEY AUTOINCREMENT, "
										 "orginal_title TEXT, "
									 	 "title TEXT, "
										 
										 "first_author INTEGER, "
										 "second_author INTEGER, "
										 "third_author INTEGER, "
										 
										 "genere INTEGER, "
										 "publish_house INTEGER, "
										 
										 "same_publish_year INTEGER, "
										 "publish_year INTEGER, "
										 
										 "same_translators INTEGER, "
										 "first_translator INTEGER, "
										 "second_translator INTEGER, "
										 "third_translator INTEGER, "
										 
										 "language INTEGER, "
										 "orginal_language INTEGER, "
										 
										 "description TEXT);", -1, &db_row, 0);								 
										 
	status = sqlite3_step (db_row);
	g_printf ("<<< Creating table \'multi_volumes\' (1/2), SQLite result: %d\n", status);
	sqlite3_finalize (db_row);
	
	status = sqlite3_prepare(database, "CREATE TABLE multi_volume_books_private_data "
										 "(id INTEGER PRIMARY KEY AUTOINCREMENT, "
										 "multi_id INTEGER, "
										 "orginal_title TEXT, "
									 	 "title TEXT, "
										 "isbn TEST, "
										 
										 "publish_year INTEGER, "
										 "pages INTEGER, "
										 
										 "first_translator INTEGER, "
										 "second_translator INTEGER, "
										 "third_translator INTEGER, "
										 
										 "description TEXT);", -1, &db_row, 0);								 
										 
	status = sqlite3_step (db_row);
	g_printf ("<<< Creating table \'multi_volumes\' (2/2), SQLite result: %d\n", status);
	sqlite3_finalize (db_row);
	
	status = sqlite3_prepare(database, "CREATE TABLE authors "
										 "(id INTEGER PRIMARY KEY AUTOINCREMENT,"
										 "author TEXT);", -1, &db_row, 0);								 
										 
	status = sqlite3_step (db_row);
	g_printf ("<<< Creating table \'authors\', SQLite result: %d\n", status);
	sqlite3_finalize (db_row);
	
	status =  sqlite3_prepare(database, "CREATE TABLE generes "
										 "(id INTEGER PRIMARY KEY AUTOINCREMENT,"
										 "genere TEXT);", -1, &db_row, 0);								 
										 
	status = sqlite3_step (db_row);
	g_printf ("<<< Creating table \'genres\', SQLite result: %d\n", status);
	sqlite3_finalize (db_row);
	
	status =  sqlite3_prepare(database, "CREATE TABLE publish_houses "
										 "(id INTEGER PRIMARY KEY AUTOINCREMENT,"
										 "publish_house TEXT);", -1, &db_row, 0);								 
										 
	status = sqlite3_step (db_row);
	g_printf ("<<< Creating table \'publish_housess\', SQLite result: %d\n", status);
	sqlite3_finalize (db_row);
	
	status =  sqlite3_prepare(database, "CREATE TABLE languages "
										 "(id INTEGER PRIMARY KEY AUTOINCREMENT,"
										 "language TEXT);", -1, &db_row, 0);								 
										 
	status = sqlite3_step (db_row);
	g_printf ("<<< Creating table \'languages\', SQLite result: %d\n", status);
	sqlite3_finalize (db_row);
	
	status =  sqlite3_prepare(database, "CREATE TABLE translators "
										 "(id INTEGER PRIMARY KEY AUTOINCREMENT,"
										 "translator TEXT);", -1, &db_row, 0);								 
										 
	status = sqlite3_step (db_row);
	g_printf ("<<< Creating table \'publishhousess\', SQLite result: %d\n", status);
	sqlite3_finalize (db_row);
	
	status =  sqlite3_prepare(database, "CREATE TABLE version "
										 "(db_version INTEGER);", -1, &db_row, 0);
										 
	status = sqlite3_step (db_row);
	g_printf ("<<< Creating table \'version\', SQLite result: %d\n", status);
	sqlite3_finalize (db_row);
	
	g_printf (">>> Inserting base data into database...\n");
	sqlite3_finalize (db_row);

	status = sqlite3_prepare(database, "INSERT INTO version (db_version) VALUES (1);", -1, &db_row, 0);
	status = sqlite3_step (db_row);
	g_printf ("\t<<< Table \'version\', SQLite result: %d\n", status);
	sqlite3_finalize (db_row);
	
	sqlite3_close (database);	
}

/*
 *	Funkcja otwiera polaczenie z baza danych
 */

void	open_db		(void)	{
	gchar	db_path	[PATH];
	gint	status;
	
	g_sprintf (db_path, "%s/.config/glibrary/database-1rc", g_get_home_dir ());
	
	status = sqlite3_open (db_path, &database);
	g_printf ("Opening data base, SQLite result: %d\n", status);
	set_statusbar_text (_("Opening database..."));
}

/*
 *	Funkcja zamyka polaczenie z baza danych
 */

void	close_db (void)	{
	gint	status;
	
	status = sqlite3_close (database);
	
	g_printf ("Closing data base, SQLite result: %d\n", status);
}

/*
 *	Funkcja sprawdza czy plik bazy danych istnieje
 */

gboolean	check_db_exist	(void)	{
	gchar		db_path [PATH];
	
	g_sprintf ( (gpointer) db_path, "%s/.config/glibrary/database-1rc", g_get_home_dir ());
	if (g_file_test ( (gpointer) db_path, G_FILE_TEST_EXISTS))
		return TRUE;
	else	{
		set_statusbar_text (_("Error: Can't find database file!"));
		return FALSE;
	}
}

/*
 *	Funkcja sprawdza, czy wersja bazy danych jest zgodna z wersja programu
 *	W GBibliotece 1.5 (oraz rozwojowej 1.4.99) baza danych ma wersje 2
 */

gboolean	check_db_version	(void)	{
	sqlite3_stmt	*db_row;
	gint			status;
	
	sqlite3_prepare (database, "SELECT * FROM version;", -1, &db_row, 0);								 
										 
	status = sqlite3_step (db_row);
	g_printf ("<<< Checking database version, SQLite result: %d\n", status);
	
	/*
	 *	Mozliwe, choc malo prawdopodobne, ze user ma baze stworzona przez 
	 *	GBiblioteke z rodziny 0.x, w bazie danych z tych wersji GBiblioteki nie ma
	 *	tabeli 'version', wiec wynik zapytania bedzie bledny, co zaraz sprawdzimy
	 */
	if (status ==! 100 || status ==! 101)	{
		set_statusbar_text (_("Error: Wrong database version!"));
		sqlite3_finalize (db_row);
		return FALSE;
	}
	
	if (sqlite3_column_int (db_row, 0) < 1)	{
		set_statusbar_text (_("Error: Wrong database version!"));
		sqlite3_finalize (db_row);
		return FALSE;
	}
	if (sqlite3_column_int (db_row, 0) > 1)	{
		set_statusbar_text (_("Error: Wrong database version!"));
		sqlite3_finalize (db_row);
		return FALSE;
	}
	if (sqlite3_column_int (db_row, 0) == 1)	{
		sqlite3_finalize (db_row);
		return TRUE;
	}
}

/*
 *	Funkcja zwraca ilosc wierszy zwroconych przez wywolanie zapytania SQL
 *	podanego w argumencie 'sql_query'.
 *	Styl obslugiwanego zapytania: 'SELECT COUNT(zmienna) FROM tabela;'
 */

gint		count_returned_rows		(gchar	*sql_query)	{
	gint			status;
	gint			result;
	sqlite3_stmt	*db_row;
	
	sqlite3_prepare (database, sql_query, -1, &db_row, 0);
	status = sqlite3_step (db_row);
	g_printf ("\t||| Counting rows, SQLite result: %d\n", status);
	
	result = sqlite3_column_int (db_row, 0);
	
	sqlite3_finalize (db_row);
	return result;
}
