/***************************************************************************
 *            main-callbacks.c
 *
 *  Fri Oct 21 22:21:39 2005
 *  Copyright  2005  Tomek Jankowski
 *  Email	tomcio-j@o2.pl
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
 
#include <glade/glade.h>
#include <gtk/gtk.h>
#include <glib.h>
#include <glib/gprintf.h>
#include <glib/gi18n-lib.h>
#include <stdio.h>

#include "main_win-callbacks.h"
#include "interface-fun.h"
#include "menage_db-fun.h"
#include "s_book-fun.h"
#include "m_book-fun.h"
#include "m_volume-fun.h"
#include "database-fun.h"
#include "search_win-callbacks.h"


gboolean
on_main_win_delete_event               (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{	
	destroy_window (widget);	
	gtk_main_quit ();
	close_db ();
}	

void
on_single_volume_activate              (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget	*widget;
	widget = create_s_book_win (0);
	gtk_widget_show (widget);
	s_prepare_book_win (widget);
}

void
on_multi_volume_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget	*widget;
	widget = create_m_book_win (0);
	gtk_widget_show (widget);
	m_prepare_book_win (widget);
}

void
on_exit_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	destroy_window (main_win);
	gtk_main_quit ();
	close_db ();
}

void
on_menage_database_activate            (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget	*widget;
	widget = create_menage_db_win ();
	gtk_widget_show (widget);
}

void
on_search_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget	*widget;
	widget = create_search_win ();
	search_preapre_win (widget);
	gtk_widget_show (widget);
}

void
on_about_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget	*widget;
	widget = create_about_dlg ();
	gtk_widget_show (widget);
}

void
on_main_add_book_but_clicked           (GtkToolButton   *toolbutton,
                                        gpointer         user_data)
{
	GtkWidget	*widget;
	widget = create_book_type_dlg ();
	gtk_widget_show (widget);
}

void
on_main_exit_but_clicked               (GtkToolButton   *toolbutton,
                                        gpointer         user_data)
{
	destroy_window (GTK_WIDGET (toolbutton));	
	gtk_main_quit ();
	close_db ();
}

void
on_main_search_en_activate             (GtkEntry        *entry,
                                        gpointer         user_data)
{
	//GtkWidget	*widget;
	//widget = create_search_win ();
	//search_preapre_win (widget);
	//gtk_widget_show (widget);
}

void
on_main_search_but_clicked             (GtkToolButton   *toolbutton,
                                        gpointer         user_data)
{
	GladeXML	*xml;
	GtkWidget	*widget;
	GtkWidget	*entry;
	
	xml = glade_get_widget_tree (GTK_WIDGET (toolbutton));
	entry = glade_xml_get_widget (xml, "main_search_en");
	/*	Otwiareamy okno wyszukiwania tylko, gdy pole 'main_search_en' nie 
	 *	jest puste	*/
	if (strcmp (gtk_entry_get_text (GTK_ENTRY (entry)), "") == 0)	{
		/*	Nic nie robimy	*/
	}
	else	{
		widget = create_search_win ();
		search_preapre_win (widget);
		gtk_widget_show (widget);
		
		xml = glade_get_widget_tree (widget);
		widget = glade_xml_get_widget (xml, "simple_search_en");
		gtk_entry_set_text (GTK_ENTRY (widget), gtk_entry_get_text (GTK_ENTRY (entry)));
		
		/*	 Zaczynamy wyszukiwanie ksiazek	*/
		widget = glade_xml_get_widget (xml, "search_search_but");
		on_search_search_but_clicked (GTK_BUTTON (widget), NULL);
	}
}


void
on_main_list_row_activated             (GtkTreeView     *treeview,
                                        GtkTreePath     *path,
                                        GtkTreeViewColumn *column,
                                        gpointer         user_data)
{
	GladeXML			*xml;
	GtkTreeSelection	*selection;
	GtkTreeModel		*model;
  	GtkTreeIter			iter;
	GtkWidget			*widget;
	gint				book_id;
	gint				book_type;
	
	xml = glade_get_widget_tree (main_win);
	widget = glade_xml_get_widget (xml, "main_notebook");
	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(treeview));
	gtk_tree_selection_get_selected (selection, &model, &iter);
	
	gtk_tree_model_get (model, &iter, 0, &book_id, -1);
	gtk_tree_model_get (model, &iter, 3, &book_type, -1);
	/*	Na wszelki wypadek czyscimy pola	*/
	s_clear_main_win ();
	m_clear_main_win ();
	m_volume_clear_main_win ();
	
	if (book_type == 0)	{
		SingleVolumeStruc	*book_data;
		
		gtk_notebook_set_current_page (GTK_NOTEBOOK (widget), 1);
		
		book_data = s_select_book_data (&book_id);
		s_display_book_data (book_data);
		
		/*	Aktywujemy odpowiednie przycisko	*/
		set_widget_sensinitive (widget, "s_remove_but", TRUE);
		set_widget_sensinitive (widget, "s_edit_but", TRUE);
	}
	else if (book_type == 1)	{
		MultiVolSharedDataStruc	*book_data;
		
		gtk_notebook_set_current_page (GTK_NOTEBOOK (widget), 2);
		
		book_data = m_select_book_data (&book_id);
		m_display_book_data (book_data);

		/*	Aktywujemy odpowiednie przycisko	*/
		set_widget_sensinitive (widget, "m_remove_but", TRUE);
		set_widget_sensinitive (widget, "m_edit_but", TRUE);
	}	
	else if (book_type == 2)	{
		MultiVolPrivDataStruc	*volume_data;
		
		gtk_notebook_set_current_page (GTK_NOTEBOOK (widget), 3);
		
		volume_data = m_volume_select_data (&book_id);
		m_volume_display_volume_data (volume_data);
		
		/*	Aktywujemy odpowiednie przycisko	*/
		set_widget_sensinitive (widget, "m_volume_remove_but", TRUE);
	}
}

void
on_main_list_cursor_changed            (GtkTreeView     *treeview,
                                        gpointer         user_data)
{
	GladeXML			*xml;
	GtkWidget			*widget;
	
	xml = glade_get_widget_tree (main_win);
	widget = glade_xml_get_widget (xml, "main_notebook");
	gtk_notebook_set_current_page (GTK_NOTEBOOK (widget), 0);
}

void
on_s_additional_informations_exp_activate (GtkExpander     *expander,
                                        gpointer         user_data)
{
	s_control_main_win_expanders (0);
}

void
on_s_description_exp_activate          (GtkExpander     *expander,
                                        gpointer         user_data)
{
	s_control_main_win_expanders (1);
}

void
on_s_edit_but_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
	GladeXML	*xml;
	GtkWidget	*tmp;
	gint		book_id;
	
	xml = glade_get_widget_tree (main_win);	
	tmp = glade_xml_get_widget (xml, "s_book_id_lb");
	sscanf (gtk_label_get_text (GTK_LABEL (tmp)), "%d", &book_id);
	
	printf ("%d\n", book_id);
	
	tmp = create_s_book_win (1);
	gtk_widget_show (tmp);
	
	s_prepare_book_win (tmp);
	s_load_edit_win (tmp, &book_id);
}

void
on_s_remove_but_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
	GladeXML	*xml;
	GtkWidget	*tmp;
	gint		book_id;
	gboolean	result;
	
	result = show_question_dlg (_("Warning!"), _("It will finally remowe book "
								"from database, do You want to continue?"), 2);
	if (result == TRUE)	{
		xml = glade_get_widget_tree (main_win);	
		tmp = glade_xml_get_widget (xml, "s_book_id_lb");
		sscanf (gtk_label_get_text (GTK_LABEL (tmp)), "%d", &book_id);
	
		s_remove_book (&book_id);
	
		/*	Przeladowujemy liste ksiazek	*/
		refresh_main_list ();
	
		/*	Czyscimy pola w oknie 'main_win'	*/
		s_clear_main_win ();
		m_clear_main_win ();
		m_volume_clear_main_win ();
	}
}

void
on_m_books_list_exp_activate           (GtkExpander     *expander,
                                        gpointer         user_data)
{
	m_control_main_win_expanders (0);
}

void
on_m_additional_informations_exp_activate  (GtkExpander     *expander,
                                        gpointer         user_data)
{
	m_control_main_win_expanders (1);
}

void
on_m_description_exp_activate          (GtkExpander     *expander,
                                        gpointer         user_data)
{
	m_control_main_win_expanders (2);
}

void
on_m_books_list_row_activated          (GtkTreeView     *treeview,
                                        GtkTreePath     *path,
                                        GtkTreeViewColumn *column,
                                        gpointer         user_data)
{
	MultiVolPrivDataStruc	*volume_data;
	GladeXML				*xml;
	GtkTreeSelection		*selection;
	GtkTreeModel			*model;
  	GtkTreeIter				iter;
	GtkWidget				*tmp;
	gint					volume_id;
	
	xml = glade_get_widget_tree (GTK_WIDGET (treeview));
	selection = gtk_tree_view_get_selection (treeview);
	gtk_tree_selection_get_selected (selection, &model, &iter);
	gtk_tree_model_get (model, &iter, 0, &volume_id, -1);
	
	tmp = create_property_dlg ();
	gtk_widget_show (tmp);
	
	volume_data = m_volume_select_data (&volume_id);
	m_volume_show_volume_data (tmp, volume_data);
}

void
on_m_edit_but_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
	MultiVolSharedDataStruc	*book_data;
	GladeXML				*xml;
	GtkWidget				*tmp;
	gint					book_id;
	
	xml = glade_get_widget_tree (GTK_WIDGET (button));
	tmp = glade_xml_get_widget (xml, "m_book_id_lb");
	sscanf (gtk_label_get_text (GTK_LABEL (tmp)), "%d", &book_id);
	
	tmp = create_m_book_win (1);
	gtk_widget_show (tmp);
	m_prepare_book_win (tmp);
	
	book_data = m_select_book_data (&book_id);
	m_load_edit_win (book_data, tmp);
	
	m_volume_load_volumes (tmp, book_id);
}

void
on_m_remove_but_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
	GladeXML	*xml;
	GtkWidget	*tmp;
	gint		book_id;
	gboolean	result;
	
	result = show_question_dlg (_("Warning!"), _("It will finally remowe book "
								"whit all volumes, which it contains. Do You want to continue?"), 2);
	if (result == TRUE)	{
		xml = glade_get_widget_tree (main_win);	
		tmp = glade_xml_get_widget (xml, "m_book_id_lb");
		sscanf (gtk_label_get_text (GTK_LABEL (tmp)), "%d", &book_id);
	
		m_remove_book (&book_id);
	
		/*	Przeladowujemy liste ksiazek	*/
		refresh_main_list ();
	
		/*	Czyscimy pola w oknie 'main_win'	*/
		s_clear_main_win ();
		m_clear_main_win ();
		m_volume_clear_main_win ();
	}
}

void
on_m_volume_additional_informations_exp_activate
                                        (GtkExpander     *expander,
                                        gpointer         user_data)
{
	m_volume_control_main_win_expanders (0);
}


void
on_m_volume_description_exp_activate   (GtkExpander     *expander,
                                        gpointer         user_data)
{
	m_volume_control_main_win_expanders (1);
}

void
on_m_volume_remove_but_clicked         (GtkButton       *button,
                                        gpointer         user_data)
{
	GladeXML	*xml;
	GtkWidget	*tmp;
	gint		book_id;
	gboolean	result;
	
	result = show_question_dlg (_("Warning!"), _("It will finally remowe book "
								"from database, do You want to continue?"), 2);
	if (result == TRUE)	{
		xml = glade_get_widget_tree (main_win);	
		tmp = glade_xml_get_widget (xml, "m_volume_book_id_lb");
		sscanf (gtk_label_get_text (GTK_LABEL (tmp)), "%d", &book_id);
	
		m_volume_remove_volume (&book_id);
	
		/*	Przeladowujemy liste ksiazek	*/
		refresh_main_list ();
	
		/*	Czyscimy pola w oknie 'main_win'	*/
		s_clear_main_win ();
		m_clear_main_win ();
		m_volume_clear_main_win ();
	}
}
