/***************************************************************************
 *            search-fun.c
 *
 *  Mon Dec 12 19:34:47 2005
 *  Copyright  2005  Tomek Jankowski
 *  Email	tomcio-j@o2.pl
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <glade/glade.h>
#include <gtk/gtk.h>
#include <glib.h>
#include <glib/gprintf.h>
#include <glib/gi18n-lib.h>
#include <stdio.h>
#include <string.h>
#include <sqlite3.h>

#include "search-fun.h"
#include "database-fun.h"
#include "book-fun.h"
#include "interface-fun.h"

#define VERY_SHORT_STRING	12
#define SHORT_STRING		128
#define MEDIUM_STRING		512
#define	LONG_STRING			1024
#define	DESCRIPTION_STRING	2048
#define	VERY_LONG_STRING	6144

void	search_preapre_win	(GtkWidget	*widget)	{
	GladeXML			*xml;
	GtkListStore		*list;
	GtkTreeIter			iter;
	GtkWidget			*tmp;
	gint				status;
	gint				rows;
	sqlite3_stmt		*db_row;
	
	xml = glade_get_widget_tree (widget);	
	g_printf (">>> Preparing window...\n");
	
	/*	Ladujemy autorow	*/
	rows = count_returned_rows ("SELECT COUNT(id) FROM authors;");
	g_printf ("\t<<< Loading %d authors...\n", rows);
	if (rows > 0)	{
		sqlite3_prepare (database, "SELECT author FROM authors;", -1, &db_row, 0);
		status = sqlite3_step (db_row);
		g_printf ("\t<<< Loading table \'authors\', SQLite result: %d\n", status);
	
		list = gtk_list_store_new (1, G_TYPE_STRING);
		if (status == 101)	{
			gtk_list_store_append (list, &iter);
			gtk_list_store_set (list, &iter,
								0, sqlite3_column_text (db_row, 0),
								-1);
		}
		else	{
			while (status == 100)	{
				gtk_list_store_append (list, &iter);
				gtk_list_store_set (list, &iter,
									0, sqlite3_column_text (db_row, 0),
									-1);
				status = sqlite3_step (db_row);
			}
		}
		
		sqlite3_finalize (db_row);
		
		/*	W koncu ladujemy listy do widgetow...	*/
		/*	Nazwa pola pierwszego autora jest inna w roznych oknach	*/
		tmp = glade_xml_get_widget (xml, "author_box");	
		gtk_combo_box_set_model (GTK_COMBO_BOX (tmp), GTK_TREE_MODEL (list));
		/*	Ustawiamy poczatkowa wartosc przycisku	*/
		gtk_combo_box_set_active (GTK_COMBO_BOX (tmp), 0);
	}
	
	/* GtkEntry 'genere_en'   */
	rows = count_returned_rows ("SELECT COUNT(id) FROM generes;");
	g_printf ("\t<<< Loading %d genres...\n", rows);
	if (rows > 0)	{
		sqlite3_prepare (database, "SELECT genere FROM generes;", -1, &db_row, 0);
		status = sqlite3_step (db_row);
		g_printf ("\t<<< Loading table \'generes\', SQLite result: %d\n", status);
		
		list = gtk_list_store_new (1, G_TYPE_STRING);
		if (status == 101)	{
			gtk_list_store_append (list, &iter);
			gtk_list_store_set (list, &iter,
								0, sqlite3_column_text (db_row, 0),
								-1);
		}
		else	{
			while (status == 100)	{
				gtk_list_store_append (list, &iter);
				gtk_list_store_set (list, &iter,
									0, sqlite3_column_text (db_row, 0),
									-1);
				status = sqlite3_step (db_row);
			}
		}
		
		sqlite3_finalize (db_row);
		
		tmp = glade_xml_get_widget (xml, "genere_box");	
		gtk_combo_box_set_model (GTK_COMBO_BOX (tmp), GTK_TREE_MODEL (list));
		/*	Ustawiamy poczatkowa wartosc przycisku	*/
		gtk_combo_box_set_active (GTK_COMBO_BOX (tmp), 0);
	}
	
	/*	Ladujemy wydawnictwa	*/
	rows = count_returned_rows ("SELECT COUNT(id) FROM publish_houses;");
	g_printf ("\t<<< Loading %d publish houses...\n", rows);
	if (rows > 0)	{
		sqlite3_prepare (database, "SELECT publish_house FROM publish_houses;", -1, &db_row, 0);
		status = sqlite3_step (db_row);
		g_printf ("\t<<< Loading table \'publish_houses\', SQLite result: %d\n", status);
		
		list = gtk_list_store_new (1, G_TYPE_STRING);
		if (status == 101)	{
			gtk_list_store_append (list, &iter);
			gtk_list_store_set (list, &iter,
								0, sqlite3_column_text (db_row, 0),
								-1);
		}
		else	{
			while (status == 100)	{
				gtk_list_store_append (list, &iter);
				gtk_list_store_set (list, &iter,
									0, sqlite3_column_text (db_row, 0),
									-1);
				status = sqlite3_step (db_row);
			}
		}
		
		sqlite3_finalize (db_row);
		
		tmp = glade_xml_get_widget (xml, "publish_house_box");	
		gtk_combo_box_set_model (GTK_COMBO_BOX (tmp), GTK_TREE_MODEL (list));
		/*	Ustawiamy poczatkowa wartosc przycisku	*/
		gtk_combo_box_set_active (GTK_COMBO_BOX (tmp), 0);
	}
	
	/*	Ladujemy jezyki	*/
	rows = count_returned_rows ("SELECT COUNT(id) FROM languages;");
	g_printf ("\t<<< Loading %d languages...\n", rows);
	if (rows > 0)	{
		sqlite3_prepare (database, "SELECT language FROM languages;", -1, &db_row, 0);
		status = sqlite3_step (db_row);
		g_printf ("\t<<< Loading table \'languages\', SQLite result: %d\n", status);
		
		list = gtk_list_store_new (1, G_TYPE_STRING);
		if (status == 101)	{
			gtk_list_store_append (list, &iter);
			gtk_list_store_set (list, &iter,
								0, sqlite3_column_text (db_row, 0),
								-1);
		}
		else	{
			while (status == 100)	{
				gtk_list_store_append (list, &iter);
				gtk_list_store_set (list, &iter,
									0, sqlite3_column_text (db_row, 0),
									-1);
				status = sqlite3_step (db_row);
			}
		}
		
		sqlite3_finalize (db_row);
		
		tmp = glade_xml_get_widget (xml, "language_box");	
		gtk_combo_box_set_model (GTK_COMBO_BOX (tmp), GTK_TREE_MODEL (list));
		/*	Ustawiamy poczatkowa wartosc przycisku	*/
		gtk_combo_box_set_active (GTK_COMBO_BOX (tmp), 0);
	}
	
	/*	Ladujemy tlumaczy	*/
	rows = count_returned_rows ("SELECT COUNT(id) FROM translators;");
	g_printf ("\t<<< Loading %d translators...\n", rows);
	if (rows > 0)	{
		sqlite3_prepare (database, "SELECT translator FROM translators;", -1, &db_row, 0);
		status = sqlite3_step (db_row);
		g_printf ("\t<<< Loading table \'translators\', SQLite result: %d\n", status);
	
		list = gtk_list_store_new (1, G_TYPE_STRING);
		if (status == 101)	{
			gtk_list_store_append (list, &iter);
			gtk_list_store_set (list, &iter,
								0, sqlite3_column_text (db_row, 0),
								-1);
		}
		else	{
			while (status == 100)	{
				gtk_list_store_append (list, &iter);
				gtk_list_store_set (list, &iter,
									0, sqlite3_column_text (db_row, 0),
									-1);
				status = sqlite3_step (db_row);
			}
		}
		
		sqlite3_finalize (db_row);
		
		/*	W koncu ladujemy listy do widgetow...	*/
		/*	Nazwa pola pierwszego tlumacza jest inna w roznych oknach	*/
		tmp = glade_xml_get_widget (xml, "translator_box");	
		gtk_combo_box_set_model (GTK_COMBO_BOX (tmp), GTK_TREE_MODEL (list));
		/*	Ustawiamy poczatkowa wartosc przycisku	*/
		gtk_combo_box_set_active (GTK_COMBO_BOX (tmp), 0);
	}
	set_widget_sensinitive (widget, "main_vbox", TRUE);
}

gboolean	search_check_search_cryterions	(GtkWidget	*widget)	{
	GladeXML	*xml;
	GtkWidget	*tmp;
	GtkWidget	*author_check;
	GtkWidget	*genere_check;
	GtkWidget	*publish_house_check;
	GtkWidget	*translator_check;
	GtkWidget	*title_check;
	GtkWidget	*language_check;
	GtkWidget	*publish_year_check;
	GtkWidget	*pages_number_check;
	gchar		*string	[LONG_STRING];
	
	xml = glade_get_widget_tree (widget);
	
	/*	Informujemy usera, ze rozpoczeto wyszukiwanie	*/
	tmp = glade_xml_get_widget (xml, "search_status_lb");
	gtk_label_set_markup (GTK_LABEL (tmp), _("<b><i>Searching...</i></b>"));
	/*	Sprawdzamy jakie wyszukiwanie jest przeprowadzane - proste czy zaawansowane	*/
	tmp = glade_xml_get_widget (xml, "addvanced_search_exp");
	/*	Jezeli GtkExpander jest zamkniety przeprowadzane jest proste wyszukiwanie	*/
	if (gtk_expander_get_expanded (GTK_EXPANDER (tmp)) == FALSE)	{
		tmp = glade_xml_get_widget (xml, "simple_search_en");
		if (strcmp (gtk_entry_get_text (GTK_ENTRY (tmp)), "") == 0)	{
			show_message_dlg (_("Wrong data!"), 
				_("Field \'Search title\' can\'t be empty!"), 1);
			return FALSE;
		}
	}
	else	{	
		author_check = glade_xml_get_widget (xml, "author_check");
		genere_check = glade_xml_get_widget (xml, "genere_check");
		publish_house_check = glade_xml_get_widget (xml, "publish_house_check");
		translator_check = glade_xml_get_widget (xml, "translator_check");
		title_check = glade_xml_get_widget (xml, "title_check");
		language_check = glade_xml_get_widget (xml, "language_check");
		publish_year_check = glade_xml_get_widget (xml, "publish_year_check");
		pages_number_check = glade_xml_get_widget (xml, "pages_number_check");
		
		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (author_check)) == FALSE&&
			gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (genere_check)) == FALSE &&
			gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (publish_house_check)) == FALSE &&
			gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (translator_check)) == FALSE &&
			gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (title_check)) == FALSE &&
			gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (language_check)) == FALSE &&
			gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (publish_year_check)) == FALSE &&
			gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (pages_number_check)) == FALSE)
		{
			show_message_dlg (_("No criterions!"), 
				_("Yuo must set some search criterions before searching."), 1);
			return FALSE;
		}
	}
	
	return TRUE;
}

/*
 *	Funkcja na podstawie kryteriow ustalonych w oknie 'search_win' wyszukuje 
 *	ksiazki jednotomowe.
 */

void		search_among_s_books		(GtkWidget	*widget)
{
	GladeXML			*xml;
	GtkWidget			*tmp;
	GtkTreeModel		*store;
	GtkTreeIter			iter;
	gchar				string		[LONG_STRING];
	gchar				sql_query	[LONG_STRING];
	gint				rows;
	gint				status;
	gint				item_id;
	gint				start_value;
	gint				end_value;
	sqlite3_stmt		*db_row;
	
	xml = glade_get_widget_tree (widget);

	/*	Sprawdzamy jakie wyszukiwanie jest przeprowadzane - proste czy zaawansowane	*/
	tmp = glade_xml_get_widget (xml, "addvanced_search_exp");
	/*	Jezeli GtkExpander jest zamkniety przeprowadzane jest proste wyszukiwanie	*/
	if (gtk_expander_get_expanded (GTK_EXPANDER (tmp)) == FALSE)	{
		tmp = glade_xml_get_widget (xml, "simple_search_en");
		g_sprintf (string, "WHERE (title LIKE \'%s%s%s\' OR orginal_title LIKE \'%s%s%s\')",
				"%",
				gtk_entry_get_text (GTK_ENTRY (tmp)), 
				"%",
				"%",
				gtk_entry_get_text (GTK_ENTRY (tmp)),
				"%");
	}
	else	{
		/*	Zaczynamy tworzenie zapytania	*/
		g_sprintf (string, "WHERE ");
		/*	Sprawdzamy wyszukiwanie po tytule	*/
		tmp = glade_xml_get_widget (xml, "title_check");
		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (tmp)) == TRUE)	{
			tmp = glade_xml_get_widget (xml, "title_en");
			/*	Doklejamy tylko nowe elementy do 'string'. Poniewaz funkcja 
			 *	'g_sprintf' nadpisuje zawartosc zmiennej za kazdym razem, 
			 *	na poczatku wklejamy poprzednia wartosc 'string'.	*/
			/*	Sprawdzamy, czy potrzebny jest operator 'AND'	*/
			if (strcmp (string, "WHERE ") ==! 0)	{
				g_sprintf (string, "%s AND ", string);
			}
			g_sprintf (string, "%s(title LIKE \'%s%s%s\' OR orginal_title LIKE \'%s%s%s\')",
				string,
				"%",
				gtk_entry_get_text (GTK_ENTRY (tmp)), 
				"%",
				"%",
				gtk_entry_get_text (GTK_ENTRY (tmp)),
				"%");
		}
		/*	Sprawdzamy wyszukiwanie po gatunku	*/
		tmp = glade_xml_get_widget (xml, "genere_check");
		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (tmp)) == TRUE)	{
			tmp = glade_xml_get_widget (xml, "genere_box");
			item_id = get_genere_id (gtk_combo_box_get_active_text (GTK_COMBO_BOX (tmp)));
			/*	Sprawdzamy, czy potrzebny jest operator 'AND'	*/
			if (strcmp (string, "WHERE ") ==! 0)	{
				g_sprintf (string, "% sAND ", string);
			}
			g_sprintf (string, "%sgenere=%d", 
				string,
				item_id);
		}
		/*	Sprawdzamy wyszukiwanie po wydawnictwie	*/
		tmp = glade_xml_get_widget (xml, "publish_house_check");
		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (tmp)) == TRUE)	{
			tmp = glade_xml_get_widget (xml, "publish_house_box");
			item_id = get_publish_house_id (gtk_combo_box_get_active_text (GTK_COMBO_BOX (tmp)));
			/*	Sprawdzamy, czy potrzebny jest operator 'AND'	*/
			if (strcmp (string, "WHERE ") ==! 0)	{
				g_sprintf (string, "%s AND ", string);
			}
			g_sprintf (string, "%spublish_house=%d", 
				string,
				item_id);
		}
		/*	Sprawdzamy wyszukiwanie po autorze	*/
		tmp = glade_xml_get_widget (xml, "author_check");
		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (tmp)) == TRUE)	{
			tmp = glade_xml_get_widget (xml, "author_box");
			item_id = get_author_id (gtk_combo_box_get_active_text (GTK_COMBO_BOX (tmp)));
			/*	Sprawdzamy, czy potrzebny jest operator 'AND'	*/
			if (strcmp (string, "WHERE ") ==! 0)	{
				g_sprintf (string, "%s AND ", string);
			}
			g_sprintf (string, 
				"%s(first_author=%d OR second_author=%d OR third_author=%d)", 
				string,
				item_id,
				item_id,
				item_id);
		}
		/*	Sprawdzamy wyszukiwanie po przedziale pomiedzy datami wydania	*/
		tmp = glade_xml_get_widget (xml, "publish_year_check");
		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (tmp)) == TRUE)	{
			tmp = glade_xml_get_widget (xml, "min_publish_year_spin");
			start_value = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (tmp));
			tmp = glade_xml_get_widget (xml, "max_publish_year_spin");
			end_value = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (tmp));			
			
			/*	Sprawdzamy, czy potrzebny jest operator 'AND'	*/
			if (strcmp (string, "WHERE ") ==! 0)	{
				g_sprintf (string, "%s AND ", string);
			}
			g_sprintf (string, "%s(publish_year BETWEEN %d AND %d)", 
				string,
				start_value,
				end_value);
		}
		/*	Sprawdzamy wyszukiwanie po przedziale pomiedzy stronami	*/
		tmp = glade_xml_get_widget (xml, "pages_number_check");
		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (tmp)) == TRUE)	{
			tmp = glade_xml_get_widget (xml, "min_pages_number_spin");
			start_value = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (tmp));
			tmp = glade_xml_get_widget (xml, "max_page_number_spin");
			end_value = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (tmp));			
			
			/*	Sprawdzamy, czy potrzebny jest operator 'AND'	*/
			if (strcmp (string, "WHERE ") ==! 0)	{
				g_sprintf (string, "%s AND ", string);
			}
			g_sprintf (string, "%s(pages BETWEEN %d AND %d)", 
				string,
				start_value,
				end_value);
		}
		/*	Sprawdzamy wyszukiwanie po tlumaczu	*/
		tmp = glade_xml_get_widget (xml, "translator_check");
		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (tmp)) == TRUE)	{
			tmp = glade_xml_get_widget (xml, "translator_box");
			item_id = get_translator_id (gtk_combo_box_get_active_text (GTK_COMBO_BOX (tmp)));
			
			/*	Sprawdzamy, czy potrzebny jest operator 'AND'	*/
			if (strcmp (string, "WHERE ") ==! 0)	{
				g_sprintf (string, "%s AND ", string);
			}
			g_sprintf (string, 
				"%s(first_translator=%d OR second_translator=%d OR third_translator=%d)", 
				string,
				item_id,
				item_id,
				item_id);
		}
		/*	Sprawdzamy wyszukiwanie po jezyku	*/
		tmp = glade_xml_get_widget (xml, "language_check");
		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (tmp)) == TRUE)	{
			tmp = glade_xml_get_widget (xml, "language_box");
			item_id = get_language_id (gtk_combo_box_get_active_text (GTK_COMBO_BOX (tmp)));
			
			/*	Sprawdzamy, czy potrzebny jest operator 'AND'	*/
			if (strcmp (string, "WHERE ") ==! 0)	{
				g_sprintf (string, "%s AND ", string);
			}
			g_sprintf (string, "%s(language=%d OR orginal_language=%d)", 
				string,
				item_id,
				item_id);
		}
	}
	
	tmp = glade_xml_get_widget (xml, "search_books_list");
	store = gtk_tree_view_get_model (GTK_TREE_VIEW (tmp));
	
	/*===	Szukamy wsrod ksiazek jednotomowych	===*/
	g_sprintf (sql_query, "SELECT COUNT(id) FROM single_volume_books %s;", string);
	//g_printf ("\n%s\n", sql_query);
	rows = count_returned_rows (sql_query);
	
	g_sprintf (sql_query, "SELECT id, title FROM single_volume_books %s;", string);
	sqlite3_prepare (database, sql_query, -1, &db_row, 0);
	status = sqlite3_step (db_row);
	g_printf ("<<< Searching among single books, SQLite result: %d\n", status);
	
	if (rows > 0)	{
		while (status == 100)		{
			gtk_list_store_append (GTK_LIST_STORE (store), &iter);
			gtk_list_store_set (GTK_LIST_STORE (store), &iter,
								0, sqlite3_column_int (db_row, 0), 
								1, sqlite3_column_text (db_row, 1),
								2, 0, /*	to pole jest ukryte	*/
								-1);
			status = sqlite3_step (db_row);
		}
	}
	sqlite3_finalize (db_row);	
}

/*
 *	Funkcja na podstawie kryteriow ustalonych w oknie 'search_win' wyszukuje 
 *	ksiazki jednotomowe.
 */

void		search_among_m_books		(GtkWidget	*widget)
{
	GladeXML			*xml;
	GtkWidget			*tmp;
	GtkTreeModel		*store;
	GtkTreeIter			iter;
	gchar				string		[LONG_STRING];
	gchar				sql_query	[LONG_STRING];
	gint				rows;
	gint				status;
	gint				item_id;
	gint				start_value;
	gint				end_value;
	sqlite3_stmt		*db_row;
	
	xml = glade_get_widget_tree (widget);

	/*	Sprawdzamy jakie wyszukiwanie jest przeprowadzane - proste czy zaawansowane	*/
	tmp = glade_xml_get_widget (xml, "addvanced_search_exp");
	/*	Jezeli GtkExpander jest zamkniety przeprowadzane jest proste wyszukiwanie	*/
	if (gtk_expander_get_expanded (GTK_EXPANDER (tmp)) == FALSE)	{
		tmp = glade_xml_get_widget (xml, "simple_search_en");
		g_sprintf (string, "WHERE (title LIKE \'%s%s%s\' OR orginal_title LIKE \'%s%s%s\')",
				"%",
				gtk_entry_get_text (GTK_ENTRY (tmp)), 
				"%",
				"%",
				gtk_entry_get_text (GTK_ENTRY (tmp)),
				"%");
	}
	else	{
		/*	Zaczynamy tworzenie zapytania	*/
		g_sprintf (string, "WHERE ");
		/*	Sprawdzamy wyszukiwanie po tytule	*/
		tmp = glade_xml_get_widget (xml, "title_check");
		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (tmp)) == TRUE)	{
			tmp = glade_xml_get_widget (xml, "title_en");
			/*	Doklejamy tylko nowe elementy do 'string'. Poniewaz funkcja 
			 *	'g_sprintf' nadpisuje zawartosc zmiennej za kazdym razem, 
			 *	na poczatku wklejamy poprzednia wartosc 'string'.	*/
			/*	Sprawdzamy, czy potrzebny jest operator 'AND'	*/
			if (strcmp (string, "WHERE ") ==! 0)	{
				g_sprintf (string, "%s AND ", string);
			}
			g_sprintf (string, "%s(title LIKE \'%s%s%s\' OR orginal_title LIKE \'%s%s%s\')",
				string,
				"%",
				gtk_entry_get_text (GTK_ENTRY (tmp)), 
				"%",
				"%",
				gtk_entry_get_text (GTK_ENTRY (tmp)),
				"%");
		}
		/*	Sprawdzamy wyszukiwanie po gatunku	*/
		tmp = glade_xml_get_widget (xml, "genere_check");
		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (tmp)) == TRUE)	{
			tmp = glade_xml_get_widget (xml, "genere_box");
			item_id = get_genere_id (gtk_combo_box_get_active_text (GTK_COMBO_BOX (tmp)));
			/*	Sprawdzamy, czy potrzebny jest operator 'AND'	*/
			if (strcmp (string, "WHERE ") ==! 0)	{
				g_sprintf (string, "% sAND ", string);
			}
			g_sprintf (string, "%sgenere=%d", 
				string,
				item_id);
		}
		/*	Sprawdzamy wyszukiwanie po wydawnictwie	*/
		tmp = glade_xml_get_widget (xml, "publish_house_check");
		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (tmp)) == TRUE)	{
			tmp = glade_xml_get_widget (xml, "publish_house_box");
			item_id = get_publish_house_id (gtk_combo_box_get_active_text (GTK_COMBO_BOX (tmp)));
			/*	Sprawdzamy, czy potrzebny jest operator 'AND'	*/
			if (strcmp (string, "WHERE ") ==! 0)	{
				g_sprintf (string, "%s AND ", string);
			}
			g_sprintf (string, "%spublish_house=%d", 
				string,
				item_id);
		}
		/*	Sprawdzamy wyszukiwanie po autorze	*/
		tmp = glade_xml_get_widget (xml, "author_check");
		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (tmp)) == TRUE)	{
			tmp = glade_xml_get_widget (xml, "author_box");
			item_id = get_author_id (gtk_combo_box_get_active_text (GTK_COMBO_BOX (tmp)));
			/*	Sprawdzamy, czy potrzebny jest operator 'AND'	*/
			if (strcmp (string, "WHERE ") ==! 0)	{
				g_sprintf (string, "%s AND ", string);
			}
			g_sprintf (string, 
				"%s(first_author=%d OR second_author=%d OR third_author=%d)", 
				string,
				item_id,
				item_id,
				item_id);
		}
		/*	Sprawdzamy wyszukiwanie po przedziale pomiedzy datami wydania	*/
		tmp = glade_xml_get_widget (xml, "publish_year_check");
		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (tmp)) == TRUE)	{
			tmp = glade_xml_get_widget (xml, "min_publish_year_spin");
			start_value = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (tmp));
			tmp = glade_xml_get_widget (xml, "max_publish_year_spin");
			end_value = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (tmp));			
			
			/*	Sprawdzamy, czy potrzebny jest operator 'AND'	*/
			if (strcmp (string, "WHERE ") ==! 0)	{
				g_sprintf (string, "%s AND ", string);
			}
			g_sprintf (string, "%s(publish_year BETWEEN %d AND %d)", 
				string,
				start_value,
				end_value);
		}
		/*	Sprawdzamy wyszukiwanie po tlumaczu	*/
		tmp = glade_xml_get_widget (xml, "translator_check");
		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (tmp)) == TRUE)	{
			tmp = glade_xml_get_widget (xml, "translator_box");
			item_id = get_translator_id (gtk_combo_box_get_active_text (GTK_COMBO_BOX (tmp)));
			
			/*	Sprawdzamy, czy potrzebny jest operator 'AND'	*/
			if (strcmp (string, "WHERE ") ==! 0)	{
				g_sprintf (string, "%s AND ", string);
			}
			g_sprintf (string, 
				"%s(first_translator=%d OR second_translator=%d OR third_translator=%d)", 
				string,
				item_id,
				item_id,
				item_id);
		}
		/*	Sprawdzamy wyszukiwanie po jezyku	*/
		tmp = glade_xml_get_widget (xml, "language_check");
		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (tmp)) == TRUE)	{
			tmp = glade_xml_get_widget (xml, "language_box");
			item_id = get_language_id (gtk_combo_box_get_active_text (GTK_COMBO_BOX (tmp)));
			
			/*	Sprawdzamy, czy potrzebny jest operator 'AND'	*/
			if (strcmp (string, "WHERE ") ==! 0)	{
				g_sprintf (string, "%s AND ", string);
			}
			g_sprintf (string, "%s(language=%d OR orginal_language=%d)", 
				string,
				item_id,
				item_id);
		}
	}
	/*	Mozliwe, ze kryteria wybrane w oknie 'search_win' nie odnosza sie do
	 *	kryteriow wyszukiwania ksiazek wielotomowych	*/
	if (strcmp (string, "WHERE ") ==! 0)	{
		tmp = glade_xml_get_widget (xml, "search_books_list");
		store = gtk_tree_view_get_model (GTK_TREE_VIEW (tmp));
		
		/*===	Szukamy wsrod ksiazek wielotomowych	===*/
		g_sprintf (sql_query, "SELECT COUNT(id) FROM multi_volume_books_shared_data %s", string);
		//g_printf ("\n%s\n\n\n", sql_query);
		rows = count_returned_rows (sql_query);
		
		g_sprintf (sql_query, "SELECT id, title FROM multi_volume_books_shared_data %s", string);
		sqlite3_prepare (database, sql_query, -1, &db_row, 0);
		status = sqlite3_step (db_row);
		g_printf ("<<< Searching books (2/3), SQLite result: %d\n", status);
		
		if (rows > 0)	{
			while (status == 100)		{
				gtk_list_store_append (GTK_LIST_STORE (store), &iter);
				gtk_list_store_set (GTK_LIST_STORE (store), &iter,
									0, sqlite3_column_int (db_row, 0), 
									1, sqlite3_column_text (db_row, 1),
									2, 1, /*	to pole jest ukryte	*/
									-1);
				status = sqlite3_step (db_row);
			}
		}
		sqlite3_finalize (db_row);
	}
}

/*
 *	Funkcja wyszukuje ksiazki wsrod tomow wchodzacych w skald ksiazek 
 *	wielotomowych na podstawie kryteriow okreslonych w oknie 'search_win'.
 */

void	search_among_m_volumes		(GtkWidget	*widget)	{
	GladeXML			*xml;
	GtkWidget			*tmp;
	GtkTreeModel		*model;
	GtkTreeIter			iter;
	gchar				string		[LONG_STRING];
	gchar				sql_query	[LONG_STRING];
	gint				rows;
	gint				status;
	gint				item_id;
	gint				start_value;
	gint				end_value;
	gboolean			result;
	sqlite3_stmt		*db_row;
	
	xml = glade_get_widget_tree (widget);

	/*	Sprawdzamy jakie wyszukiwanie jest przeprowadzane - proste czy zaawansowane	*/
	tmp = glade_xml_get_widget (xml, "addvanced_search_exp");
	/*	Jezeli GtkExpander jest zamkniety przeprowadzane jest proste wyszukiwanie	*/
	if (gtk_expander_get_expanded (GTK_EXPANDER (tmp)) == FALSE)	{
		tmp = glade_xml_get_widget (xml, "simple_search_en");
		g_sprintf (string, "WHERE (multi_volume_books_private_data.title LIKE "
				"\'%s%s%s\' OR multi_volume_books_private_data.orginal_title LIKE \'%s%s%s\')",
				"%",
				gtk_entry_get_text (GTK_ENTRY (tmp)), 
				"%",
				"%",
				gtk_entry_get_text (GTK_ENTRY (tmp)),
				"%");
	}
	else	{
		/*	Zaczynamy tworzenie zapytania	*/
		g_sprintf (string, "WHERE ");
		/*	Sprawdzamy wyszukiwanie po tytule	*/
		tmp = glade_xml_get_widget (xml, "title_check");
		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (tmp)) == TRUE)	{
			tmp = glade_xml_get_widget (xml, "title_en");
			/*	Doklejamy tylko nowe elementy do 'string'. Poniewaz funkcja 
			 *	'g_sprintf' nadpisuje zawartosc zmiennej za kazdym razem, 
			 *	na poczatku wklejamy poprzednia wartosc 'string'.	*/
			/*	Sprawdzamy, czy potrzebny jest operator 'AND'	*/
			if (strcmp (string, "WHERE ") ==! 0)	{
				g_sprintf (string, "%s AND ", string);
			}
			g_sprintf (string, "%s(multi_volume_books_private_data.title LIKE "
				"\'%s%s%s\' OR multi_volume_books_private_data.orginal_title LIKE \'%s%s%s\')",
				string,
				"%",
				gtk_entry_get_text (GTK_ENTRY (tmp)), 
				"%",
				"%",
				gtk_entry_get_text (GTK_ENTRY (tmp)),
				"%");
		}
		/*	Sprawdzamy wyszukiwanie po gatunku	*/
		tmp = glade_xml_get_widget (xml, "genere_check");
		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (tmp)) == TRUE)	{
			tmp = glade_xml_get_widget (xml, "genere_box");
			item_id = get_genere_id (gtk_combo_box_get_active_text (GTK_COMBO_BOX (tmp)));
			/*	Sprawdzamy, czy potrzebny jest operator 'AND'	*/
			if (strcmp (string, "WHERE ") ==! 0)	{
				g_sprintf (string, "% sAND ", string);
			}
			g_sprintf (string, "%smulti_volume_books_shared_data.genere=%d", 
				string,
				item_id);
		}
		/*	Sprawdzamy wyszukiwanie po wydawnictwie	*/
		tmp = glade_xml_get_widget (xml, "publish_house_check");
		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (tmp)) == TRUE)	{
			tmp = glade_xml_get_widget (xml, "publish_house_box");
			item_id = get_publish_house_id (gtk_combo_box_get_active_text (GTK_COMBO_BOX (tmp)));
			/*	Sprawdzamy, czy potrzebny jest operator 'AND'	*/
			if (strcmp (string, "WHERE ") ==! 0)	{
				g_sprintf (string, "%s AND ", string);
			}
			g_sprintf (string, "%smulti_volume_books_shared_data.publish_house=%d", 
				string,
				item_id);
		}
		/*	Sprawdzamy wyszukiwanie po autorze	*/
		tmp = glade_xml_get_widget (xml, "author_check");
		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (tmp)) == TRUE)	{
			tmp = glade_xml_get_widget (xml, "author_box");
			item_id = get_author_id (gtk_combo_box_get_active_text (GTK_COMBO_BOX (tmp)));
			/*	Sprawdzamy, czy potrzebny jest operator 'AND'	*/
			if (strcmp (string, "WHERE ") ==! 0)	{
				g_sprintf (string, "%s AND ", string);
			}
			g_sprintf (string, 
				"%s(multi_volume_books_shared_data.first_author=%d OR "
				"multi_volume_books_shared_data.second_author=%d OR "
				"multi_volume_books_shared_data.third_author=%d)", 
				string,
				item_id,
				item_id,
				item_id);
		}
		/*	Sprawdzamy wyszukiwanie po przedziale pomiedzy datami wydania	*/
		tmp = glade_xml_get_widget (xml, "publish_year_check");
		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (tmp)) == TRUE)	{
			tmp = glade_xml_get_widget (xml, "min_publish_year_spin");
			start_value = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (tmp));
			tmp = glade_xml_get_widget (xml, "max_publish_year_spin");
			end_value = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (tmp));			
			
			/*	Sprawdzamy, czy potrzebny jest operator 'AND'	*/
			if (strcmp (string, "WHERE ") ==! 0)	{
				g_sprintf (string, "%s AND ", string);
			}
			g_sprintf (string, "%s(multi_volume_books_private_data.publish_year BETWEEN %d AND %d)", 
				string,
				start_value,
				end_value);
		}
		/*	Sprawdzamy wyszukiwanie po przedziale pomiedzy stronami	*/
		tmp = glade_xml_get_widget (xml, "pages_number_check");
		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (tmp)) == TRUE)	{
			tmp = glade_xml_get_widget (xml, "min_pages_number_spin");
			start_value = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (tmp));
			tmp = glade_xml_get_widget (xml, "max_page_number_spin");
			end_value = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (tmp));			
			
			/*	Sprawdzamy, czy potrzebny jest operator 'AND'	*/
			if (strcmp (string, "WHERE ") ==! 0)	{
				g_sprintf (string, "%s AND ", string);
			}
			g_sprintf (string, "%s(multi_volume_books_private_data.pages BETWEEN %d AND %d)", 
				string,
				start_value,
				end_value);
		}
		/*	Sprawdzamy wyszukiwanie po tlumaczu	*/
		tmp = glade_xml_get_widget (xml, "translator_check");
		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (tmp)) == TRUE)	{
			tmp = glade_xml_get_widget (xml, "translator_box");
			item_id = get_translator_id (gtk_combo_box_get_active_text (GTK_COMBO_BOX (tmp)));
			
			/*	Sprawdzamy, czy potrzebny jest operator 'AND'	*/
			if (strcmp (string, "WHERE ") ==! 0)	{
				g_sprintf (string, "%s AND ", string);
			}
			g_sprintf (string, 
				"%s(multi_volume_books_private_data.first_translator=%d OR "
				"multi_volume_books_private_data.second_translator=%d OR "
				"multi_volume_books_private_data.third_translator=%d)", 
				string,
				item_id,
				item_id,
				item_id);
		}
		/*	Sprawdzamy wyszukiwanie po jezyku	*/
		tmp = glade_xml_get_widget (xml, "language_check");
		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (tmp)) == TRUE)	{
			tmp = glade_xml_get_widget (xml, "language_box");
			item_id = get_language_id (gtk_combo_box_get_active_text (GTK_COMBO_BOX (tmp)));
			
			/*	Sprawdzamy, czy potrzebny jest operator 'AND'	*/
			if (strcmp (string, "WHERE ") ==! 0)	{
				g_sprintf (string, "%s AND ", string);
			}
			g_sprintf (string, "%s(multi_volume_books_shared_data.language=%d OR "
				"multi_volume_books_shared_data.orginal_language=%d)", 
				string,
				item_id,
				item_id);
		}
	}
	
	/*===	Szukamy wsrod tomow	===*/
	g_sprintf (sql_query, "SELECT COUNT(multi_volume_books_private_data.id) "
		"FROM multi_volume_books_private_data, multi_volume_books_shared_data %s;", string);
	//g_printf ("\n%s\n\n\n", sql_query);
	rows = count_returned_rows (sql_query);
	if (rows > 0)	{
		g_sprintf (sql_query, "SELECT "
			"multi_volume_books_private_data.id, "
			"multi_volume_books_shared_data.title, "
			"multi_volume_books_private_data.title "
			"FROM multi_volume_books_private_data, multi_volume_books_shared_data %s;", string);
		//g_printf ("\n%s\n", sql_query);
		sqlite3_prepare (database, sql_query, -1, &db_row, 0);
		status = sqlite3_step (db_row);
		g_printf ("<<< Searching among volumes, SQLite result: %d\n", status);
		tmp = glade_xml_get_widget (xml, "search_books_list");
		model = gtk_tree_view_get_model (GTK_TREE_VIEW (tmp));
		
		while (status == 100)		{
			g_sprintf (string, "%s - %s", 
				sqlite3_column_text (db_row, 1),
				sqlite3_column_text (db_row, 2));
				gtk_list_store_append (GTK_LIST_STORE (model), &iter);
				gtk_list_store_set (GTK_LIST_STORE (model), &iter,
								0, sqlite3_column_int (db_row, 0), 
								1, string,
								2, 2, /*	to pole jest ukryte	*/
								-1);
			status = sqlite3_step (db_row);
		}
		sqlite3_finalize (db_row);
	}
}

/*
 *	Funkcja liczy znalezione ksiazki i informuje o tym uzytkownika oraz
 *	uaktywnia liste 'search_books_list'.
 */

void	search_count_books		(GtkWidget	*widget)	{
	GladeXML			*xml;
	GtkTreeModel		*model;
	GtkTreeIter			iter;
	GtkWidget			*tmp;
	gint				rows;
	gchar				string	[SHORT_STRING];
	gboolean			result;
	
	xml = glade_get_widget_tree (widget);
	rows = 0;
	
	tmp = glade_xml_get_widget (xml, "search_books_list");
	model = gtk_tree_view_get_model (GTK_TREE_VIEW (tmp));
	
	result = gtk_tree_model_get_iter_first (model, &iter);
	while (result == TRUE)	{		
		rows++;
		/*	Pobieramy nastepny wiersz	*/
		result = gtk_tree_model_iter_next (model, &iter);
	}
	
	tmp = glade_xml_get_widget (xml, "search_status_lb");
	g_sprintf (string, _("<b><i>Found %d books</i></b>"), rows);
	gtk_label_set_markup (GTK_LABEL (tmp), string);
	
	/*	Uaktywniamy listen*/
	set_widget_sensinitive (tmp, "search_books_list", TRUE);
}
