/***************************************************************************
 *            search_win-callbacks.c
 *
 *  Sat Oct 22 10:41:26 2005
 *  Copyright  2005  Tomek Jankowski
 *  Email	tomcio-j@o2.pl
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <glade/glade.h>
#include <gtk/gtk.h>
#include <glib.h>
#include <glib/gprintf.h>
#include <glib/gi18n-lib.h>
#include <stdio.h>

#include "search_win-callbacks.h"
#include "interface-fun.h"
#include "search-fun.h"
#include "s_book-fun.h"
#include "m_book-fun.h"
#include "m_volume-fun.h"

gboolean
on_search_win_delete_event             (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
	destroy_window (widget);
	return FALSE;
}

void
on_addvanced_search_exp_activate       (GtkExpander     *expander,
                                        gpointer         user_data)
{
	if (gtk_expander_get_expanded (expander) == TRUE)
		set_widget_sensinitive (GTK_WIDGET (expander), "simple_search_hbox", TRUE);
	else
		set_widget_sensinitive (GTK_WIDGET (expander), "simple_search_hbox", FALSE);
}

void
on_title_check_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button)) == TRUE)
		set_widget_sensinitive (GTK_WIDGET (button), "title_en", TRUE);
	else
		set_widget_sensinitive (GTK_WIDGET (button), "title_en", FALSE);
}

void
on_genere_check_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button)) == TRUE)
		set_widget_sensinitive (GTK_WIDGET (button), "genere_box", TRUE);
	else
		set_widget_sensinitive (GTK_WIDGET (button), "genere_box", FALSE);
}

void
on_publish_house_check_clicked         (GtkButton       *button,
                                        gpointer         user_data)
{
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button)) == TRUE)
		set_widget_sensinitive (GTK_WIDGET (button), "publish_house_box", TRUE);
	else
		set_widget_sensinitive (GTK_WIDGET (button), "publish_house_box", FALSE);
}

void
on_language_check_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button)) == TRUE)
		set_widget_sensinitive (GTK_WIDGET (button), "language_box", TRUE);
	else
		set_widget_sensinitive (GTK_WIDGET (button), "language_box", FALSE);
}

void
on_publish_year_check_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button)) == TRUE)
		set_widget_sensinitive (GTK_WIDGET (button), "pub_year_hbox", TRUE);
	else
		set_widget_sensinitive (GTK_WIDGET (button), "pub_year_hbox", FALSE);
}

void
on_pages_number_check_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button)) == TRUE)
		set_widget_sensinitive (GTK_WIDGET (button), "pages_num_hbox", TRUE);
	else
		set_widget_sensinitive (GTK_WIDGET (button), "pages_num_hbox", FALSE);
}

void
on_author_check_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button)) == TRUE)
		set_widget_sensinitive (GTK_WIDGET (button), "author_box", TRUE);
	else
		set_widget_sensinitive (GTK_WIDGET (button), "author_box", FALSE);
}


void
on_translator_check_clicked            (GtkButton       *button,
                                        gpointer         user_data)
{
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button)) == TRUE)
		set_widget_sensinitive (GTK_WIDGET (button), "translator_box", TRUE);
	else
		set_widget_sensinitive (GTK_WIDGET (button), "translator_box", FALSE);
}

void
on_search_books_list_row_activated     (GtkTreeView     *treeview,
                                        GtkTreePath     *path,
                                        GtkTreeViewColumn *column,
                                        gpointer         user_data)
{
	MultiVolPrivDataStruc	*volume_data;
	GladeXML			*xml;
	GtkTreeSelection	*selection;
	GtkTreeModel		*model;
  	GtkTreeIter			iter;
	GtkWidget			*widget;
	gint				book_id;
	gint				book_type;
	
	xml = glade_get_widget_tree (main_win);
	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(treeview));
	gtk_tree_selection_get_selected (selection, &model, &iter);
	
	gtk_tree_model_get (model, &iter, 0, &book_id, 2, &book_type, -1);
	
	widget = create_property_dlg ();
	switch (book_type)	{
		case 0:
			s_show_book_data (widget, &book_id);
			break;
		case 1:		
			m_show_book_data (widget, &book_id);
			break;
		case 2:			
			volume_data = m_volume_select_data (&book_id);
			m_volume_show_volume_data (widget, volume_data);
			break;
		default:
			break;
	}
	
	gtk_widget_show (widget);
}

void
on_search_search_but_clicked           (GtkButton       *button,
                                        gpointer         user_data)
{
	GladeXML	*xml;
	GtkWidget	*tmp;
	gboolean	result;
	gchar		*string;
	
	/*	Czyscimy liste przed wyszukiwaniem	*/
	xml = glade_get_widget_tree (GTK_WIDGET (button));
	tmp = glade_xml_get_widget (xml, "search_books_list");
	clear_list_list (tmp);
	
	result = search_check_search_cryterions (GTK_WIDGET (button));
	if (result == TRUE)	{
		gtk_widget_hide (tmp);
		
		search_among_s_books (GTK_WIDGET (button));
		search_among_m_books (GTK_WIDGET (button));
		search_among_m_volumes (GTK_WIDGET (button));
		search_count_books (GTK_WIDGET (button));
		
		gtk_widget_show (tmp);
	}
}

void
on_search_stop_but_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{
	
}

void
on_search_close_but_clicked            (GtkButton       *button,
                                        gpointer         user_data)
{
	destroy_window (GTK_WIDGET (button));
}
